/* ali.c common function for mfe and partition function */
#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <ctype.h>
#include <errno.h>
#include <string.h>
#include <config.h>
#include "ali.h"

#define PRIVATE static

PRIVATE void add_to_pairtable(short *table, const char *structure,
			      char L, char R);

void print_pi(pair_info pi) {
  char *pname[8] = {"","CG","GC","GU","UG","AU","UA", "--"};
  int i;
  
  /* numbering starts with 1 in output */
  if (pi.i > pi.j)  return;
  if (pi.p<0.01) return;  /* don't print entries with mean p<1% */
  printf("%5d %5d %2d %5.1f%% %7.3f",
	 pi.i+1, pi.j+1, pi.bp[0], 100*pi.p, pi.ent);   
  for (i=1; i<=7; i++) 
    if (pi.bp[i]) printf(" %s:%-4d", pname[i], pi.bp[i]);
  /* if ((!pi.sym)&&(pi.j>=0)) printf(" *"); */
  if (pi.comp!=1) printf(" +");
  printf("\n");
}
   
/*-------------------------------------------------------------------------*/

int read_clustal(FILE *clust, char *AlignedSeqs[], char *names[]) {
  char *line, workstr[100], *strg;
  int i, j, n, nn;
  
  line=get_line(clust);
  sscanf(line,"%99s",workstr);
  
  if (strcmp(workstr,"CLUSTAL") !=0) {
    fprintf(stderr,"Sorry. Expecting CLUSTAL file.\n");
    return 0;
  }
  free(line); 
  line = get_line(clust); 
  while (line && strlen(line)<4) {
    free(line); /* skip empty line */
    line = get_line(clust); 
  } 
  
  nn=0;
  while (((n=strlen(line))>3) && !isspace((int)line[0]) ) {

    strg = (char *) space( (n+1)*sizeof(char) );
    sscanf(line,"%99s %s",workstr,strg);
    names[nn] = strdup(workstr);
    AlignedSeqs[nn] = strdup(strg);
    free(strg);
    free(line);
    line = get_line(clust);
    nn++;
  }
  
  /* ignore the consensus-line for the moment */
  free(line);
  line = get_line(clust);  
  
  while(line!=NULL) {    
    while (((n=strlen(line))<4) || isspace((int)line[0])){
      free(line);
      line = get_line(clust); 
    } 
    
    nn=0;
    while ((n>3) && !isspace((int)line[0])) {

      strg = (char *) space( (n+1)*sizeof(char) );
      sscanf(line,"%s %s",workstr,strg);
      if (strcmp(workstr,names[nn])==0) {
	AlignedSeqs[nn] = (char *)
	  realloc(AlignedSeqs[nn], 
		  strlen(strg)+strlen(AlignedSeqs[nn])+1);
	strcat(AlignedSeqs[nn],strg); 
	nn++;
      } else {
	fprintf(stderr, "Sorry, your file is fucked up.\n");
	return 0;
      }
      free(strg);
      free(line);
      line = get_line(clust);
    }
    /* ignore the consensus-line for the moment */
    free(line);
    line = get_line(clust);      
  }   
  
  n = strlen(AlignedSeqs[0]); 
  for (i=1; i<nn; i++) {
    if (strlen(AlignedSeqs[i])!=n) {
      fprintf(stderr, "Sorry, your file is fucked up.\n"
	      "Unequal lengths!\n\n");
      return 0;
    }
  }
  
  for (i=0; i<nn; i++) 
    for (j=0; j<n; j++)
      if (AlignedSeqs[i][j]=='T') AlignedSeqs[i][j]='U';
  printf("%d sequences; length of alignment %d.", nn, n);
  AlignedSeqs[nn] = names[nn] = NULL;
  return nn;
}

/*-------------------------------------------------------------------------*/

 
char *get_line(FILE *fp) /* reads lines of arbitrary length from fp */
{
   char s[512], *line, *cp;
 
   line = NULL;
   do {
      if (fgets(s, 512, fp)==NULL) break;
      cp = strchr(s, '\n');
      if (cp != NULL) *cp = '\0';
      if (line==NULL)
         line = space(strlen(s)+1);
      else
         line = (char *) realloc(line, strlen(s)+strlen(line)+1);
      strcat(line,s);
   } while(cp==NULL);
 
   return line;
}
 
/*-------------------------------------------------------------------------*/
#undef space
void *space(unsigned size)
{
  void *pointer;
  
  if ( (pointer = (void *) calloc(1, size)) == NULL) {
#ifdef EINVAL
    if (errno==EINVAL) {
      fprintf(stderr,"SPACE: requested size: %d\n", size);
      nrerror("SPACE allocation failure -> EINVAL");
    }
    if (errno==ENOMEM)
#endif
      nrerror("SPACE allocation failure -> no memory");
  }
  return  pointer;
}
#ifdef WITH_DMALLOC
#define space(S) calloc(1,(S))
#endif

#undef xrealloc
/* dmalloc.h #define's xrealloc */
void *xrealloc (void *p, unsigned size) {
  if (p == 0)
    return space(size);
  p = (void *) realloc(p, size);
  if (p == NULL) {
#ifdef EINVAL
    if (errno==EINVAL) {
      fprintf(stderr,"xrealloc: requested size: %d\n", size);
      nrerror("xrealloc allocation failure -> EINVAL");
    }
    if (errno==ENOMEM)
#endif
      nrerror("xrealloc allocation failure -> no memory");  
  }
  return p;
}

/*------------------------------------------------------------------------*/
 

void nrerror(char *message)       /* print error message and die */
{
  fprintf(stderr, "\n%s\n", message);
  exit(-1);
}

/*------------------------------------------------------------------------*/

short *make_pair_table(const char *structure)
{
    /* returns array representation of structure.
       table[i] is 0 if unpaired or j if (i.j) pair.  */
   short i,b,length;
   short *table;
   
   length = (short) strlen(structure);

   table = (short *) space(sizeof(short)*(length+2));
   table[0] = length;

   for (i=1;i<=length;i++) {
     switch(structure[i-1]) {
     case '.' :  /* unpaired base */
     case ' ' :  /* all sorts of gap characters */
     case '-' :
     case '_' :
     case '=' :
     case '|' :
     case '~' :
       table[i]=0; break;
     default :
       table[i]=-1;  /* mark as not yet read */
       break;
     }
   }
   add_to_pairtable(table,structure,'(',')');

   for (b=0,i=1;i<=length;i++) {
     if (table[i]<0) b=1;
   }
   if (!b) return(table);

   fprintf(stderr,"Input may contain pseudoknots.\n");

   /* Besides '()' we allow other parentheses for pseudoknots */
   add_to_pairtable(table,structure,'[',']');
   add_to_pairtable(table,structure,'<','>');
   add_to_pairtable(table,structure,'{','}');
   
   for (b=0,i=1;i<=length;i++) {
     if (table[i]<0) {
       b=1;
       table[i]=0;
     }
   }
   if (!b) return(table);

   fprintf(stderr,
	   "Warning. Ignoring undefinded characters in input structure.\n");

   return table;
}

PRIVATE void add_to_pairtable(short *table, const char *structure,
			      char L, char R){
  int i, j,hx;
  short *stack;
  short length;

  length = strlen(structure);
  stack = (short *) space(sizeof(short)*(length+1));  
  for (hx=0, i=1; i<=length; i++) {
    if (structure[i-1]==L) { 
      stack[hx++]=i;
    }
    else if (structure[i-1]==R) {
      j = stack[--hx];
      if (hx<0) {
	fprintf(stderr, "%s\nj=%d ,%c\n", structure,j,R);
	nrerror("unbalanced brackets in make_pair_table");
      }
      table[i]=j;
      table[j]=i;
    }
  }
  if (hx!=0) {
    fprintf(stderr, "%s\n", structure);
    nrerror("unbalanced brackets (end of sequence) in make_pair_table");
  }
  free(stack);
}
/* End of file */
