/* Last changed Time-stamp: <2004-06-18 17:13:57 ivo> */
/* various comparison functions for the rank-ordering base pair lists by credibility */

#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <ctype.h>
#include <string.h>
#include <config.h>
#include "ali.h"


/* all these functions can be used to sort a *pair_info list using qsort */
extern struct options {
  int want_quiet, want_verbose;
  int p;
  int compf;
  float pfrali_cw;
  float covar_nc;
  float covar_beta;
  char ext[16];
  FILE *INFILE;
} alidot_opt;


int compare_pi_pfrali(const void *pi1, const void *pi2) {
  /* the "classic" pfrali comparison function */
   pair_info *p1, *p2;
   int i, nc1, nc2;
   float pp1, pp2;
   
   p1 = (pair_info *) pi1;
   p2 = (pair_info *) pi2;

   /* j<0 indicates i is unpaired, not used for pf */
   if (p1->j*p2->j<0) return p2->j - p1->j; 

   /* first criterium: number of incompatible sequs */
   if (p1->bp[0]!=p2->bp[0]) return (p1->bp[0]-p2->bp[0]);

   /* no longer used */
   /* if ((p2->sym - p1->sym)!=0) return (p2->sym - p1->sym); */

   /* next criterium: number of compensatory mutations */
   for (nc1=nc2=0, i=1; i<=6; i++) {
      if (p1->bp[i]>0) nc1++;
      if (p2->bp[i]>0) nc2++;
   }

   /* if there is just 1 mutation: consistent is worse then compensatory */
   if (nc1==2)
     if (p1->bp[3]*(p1->bp[2]+p1->bp[5]) + p1->bp[4]*(p1->bp[1]+p1->bp[6]))
       nc1=alidot_opt.pfrali_cw;
   if (nc2==2)
     if (p2->bp[3]*(p2->bp[2]+p2->bp[5]) + p2->bp[4]*(p2->bp[1]+p2->bp[6]))
       nc2=alidot_opt.pfrali_cw;

   pp1 = nc1*p1->p;
   pp2 = nc2*p2->p;
   if (pp1 < pp2) return 1;
   else {
      if (pp1 == pp2) return (p1->i - p2->i);
      else return -1;
   }
}


int compare_pi_alidot(const void *pi1, const void *pi2) {
  /* the classic alidot compare function */
   pair_info *p1, *p2;
   int i, nc1, nc2;
   
   p1 = (pair_info *) pi1;
   p2 = (pair_info *) pi2;

   if (p1->j*p2->j<0) return p2->j - p1->j;  /* negative j comes last */
   /* less incompatibles is better */
   if (p1->bp[0]!=p2->bp[0]) return (p1->bp[0]-p2->bp[0]);
   /* symetric is bettter */    /* no longer used */
   /* if ((p2->sym - p1->sym)!=0) return (p2->sym - p1->sym); */
   /* more compensatory mutations is better */
   for (nc1=nc2=0, i=1; i<=6; i++) {
     if (p1->bp[i]>0) nc1++;
     if (p2->bp[i]>0) nc2++;
   }

   if (nc1 != nc2) return (nc2-nc1);

   /* if there is just 1 mutation: compensatory is better then consistent */
   if (nc1==2) {
     int con1, con2;
     con1 = p1->bp[3]*(p1->bp[2]+p1->bp[5]) + p1->bp[4]*(p1->bp[1]+p1->bp[6]);
     con2 = p2->bp[3]*(p2->bp[2]+p2->bp[5]) + p2->bp[4]*(p2->bp[1]+p2->bp[6]);
     if (con1 != con2) return (con1 - con2);
   }

   /* smaller entropy is better */
   if (p1->ent > p2->ent) return 1;
   else {
      if (p1->ent == p2->ent) return (p1->i - p2->i);
      else return -1;
   }
}

int compare_pi_cov(const void *pi1, const void *pi2) {
  /* the covariance based compare function (as in alifold) */
   pair_info *p1, *p2;
   int k, l, N;
   double score1, score2;
   int dm[7][7]={{0,0,0,0,0,0,0}, /* hamming distance between pairs */
                {0,0,2,2,1,2,2} /* CG */,
                {0,2,0,1,2,2,2} /* GC */,
                {0,2,1,0,2,1,2} /* GU */,
                {0,1,2,2,0,2,1} /* UG */,
                {0,2,2,1,2,0,2} /* AU */,
                {0,2,2,2,1,2,0} /* UA */};

   p1 = (pair_info *) pi1;
   p2 = (pair_info *) pi2;

   for (N=k=0; k<=7; k++) N += p1->bp[k];
   
   for (k=1,score1=0; k<=6; k++) /* ignore pairtype 7 (gap-gap) */
     for (l=k+1; l<=6; l++) 
       score1 += p1->bp[k]*p1->bp[l]*dm[k][l];
   for (k=1,score2=0; k<=6; k++) /* ignore pairtype 7 (gap-gap) */
     for (l=k+1; l<=6; l++)
       score2 += p2->bp[k]*p2->bp[l]*dm[k][l];
   
   score1 = score1/N - (p1->bp[0] + 0.25*p1->bp[7])*alidot_opt.covar_nc;
   score2 = score2/N - (p2->bp[0] + 0.25*p2->bp[7])*alidot_opt.covar_nc;

   score1 += N*alidot_opt.covar_beta*log(p1->p);
   score2 += N*alidot_opt.covar_beta*log(p2->p);

   if (score1 == score2) return 0;
   return (score1 > score2)? -1:1;
}
   
/* End of file */
