#!/usr/bin/perl -w
# zoom into a PostScript dot plot produced by alidot or RNAfold
# and rotate by 45 degrees, producing horizontal bands along the diagonal.
# Last changed Time-stamp: <2003-03-24 13:18:20 ivo> 
sub usage {
    die "Usage: $0 [-f first] [-l last] [aln_dp.ps]\n";
}

$from=1;
$to = -1;
init();

while (<>) { #scan until definition of sequence
    if (/\/sequence \{ \((\S*)[\\\)]/) {
        $seq = $1;              # empty for new version
        while (!/\) \} def/) {  # read until end of definition
            $_ = <>;
            /(\S*)[\\\)]/;      # ends either with `)' or `\'
            $seq .= $1;
        }
        if ($to>$from) { $seq=substr($seq,$from-1,$to-$from+1) }
        else { $seq=substr($seq,$from-1) }
        $seq =~ s/\(/\\(/g;  # quote brackets
        $seq =~ s/\)/\\)/g;
	print  "% Subsequence from $from to $to\n";
	print "/sequence { ($seq) } def\n";
    }
    elsif (/^%%BoundingBox/) { #replace BoundingBox
	print "%%BoundingBox:38 24 535 836\n%%Orientation: Landscape\n";
    }
    elsif (/\d+ \d+ translate/) { # replace the rest of the header
	print $header;
	last;
    }
    elsif (/%delete next line/) {
	$_ = <>;  # skip next line
    }
    else {
	print;    # copy other header lines
    }
}

while (<>) {
    if ( /(\d+) (\d+) (.*box$)/ ) {
	$i=$1; $j=$2;
	next if ((($i+$j)<2*$from) || ((($i+$j)>2*$to)&&($to>0)));
	$i -= ($from-1);
	$j -= ($from-1);
	printf "%d %d %s\n", $i, $j, $3;
    }
}
print "showpage\n";

sub init {
    while ($_ = shift(@ARGV)) {
	/^-[\?h]/ && &usage;
	/^-f$/ && ($from = shift(@ARGV), next );
	/^-l$/ && ($to   = shift(@ARGV), next );
	unshift (@ARGV, $_);
	last;
    }
    usage() if eof();

    $header = <<EOF;
% Start modified header inserted by dprzoom
0 20 translate
298 421 moveto	%moveto center of page
1.3 dup scale
135 rotate
% move currentpoint back to center of page
currentpoint 421 sub exch 298 sub exch translate


72 216 translate
72 6 mul len div dup scale
/Times-Roman findfont 0.95 scalefont setfont

% draw diagonal
0.04 setlinewidth
0.5 len 0.5 add moveto 0.5 len add 0.5 lineto stroke 


0.01 setlinewidth
/gridsize { % grid spacing
   len log 1 sub cvi 10 exch exp
} def  
gridsize dup 20 div dup 2 array astore exch 40 div setdash

/h { len 3 div gridsize div cvi gridsize mul} def

%set a clipping path
0.5 dup translate
newpath
h neg len h sub moveto
h len h add lineto
len h add h lineto
len h sub h neg lineto
closepath
clip
0.5 neg dup translate

%/h 0 def
gridsize 0.5 h sub exch  0.5 len h add add {
% for (i=0.5; i<=len+0.5; i+=spacing)
   dup dup
   0.5 h sub moveto         % i 0.5 moveto
   len 0.5 add h add lineto      % i len+0.5 lineto
   dup
   len 1 add exch sub 0.5 h sub exch moveto          % 0.5 i moveto
   len 0.5 add h add exch len 1 add exch sub lineto  % len+0.5 i lineto
} for
stroke
% End dprzoom header
EOF

}
