/*
                 ALIDOT -- conserved RNA structure motifs
                                   
                 c Ivo L Hofacker and Peter F Stadler
                          Vienna RNA package
*/
/*Last changed Time-stamp: <2004-06-18 17:15:40 ivo> */
#include <config.h>
#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <ctype.h>
#include <string.h>
#include <getopt.h>
#include "pair_mat.h"
#include "ali.h"
#include "compare.h"
#include "pair_stuff.h"

static char rcsid[] = "$Id: main.c,v 1.10 2004/06/18 15:25:37 ivo Exp $";

#define PUBLIC
#define PRIVATE     static

#define PTHRESH  0.2

typedef struct {
   int i,j;      /* (i.j)  original coords (starting at 1!)*/
   unsigned short ai, aj;   /* (i.j) in alignment coords (starting at 0!) */
   float p;      /* probability */
} BasePair;

PRIVATE BasePair *read_bppm(char *seqname, char *ext, char *sequ);
PRIVATE BasePair *read_lfold(char *seqname, char *ext, char *sequ);
PRIVATE int (* compare_pair_info)(const void *pi1, const void *pi2);
PRIVATE int comp_BasePair(const void *pi1, const void *pi2);
PRIVATE int decode_switches (int argc, char **argv);
PRIVATE  pair_info* get_pf_bpairs(char *the_names[], char *AS[]);
PRIVATE pair_info* get_mfe_bpairs(char *the_names[], char *AS[]);
PRIVATE pair_info* get_lfold_bpairs(char *the_names[], char *AS[]);
PRIVATE void usage(int code);
PRIVATE char *read_structure(char *seqname, char *ext);
PRIVATE char *expand_struct(char *sequ, char *struc);

PRIVATE struct option const long_options[] =
{
  {"quiet", no_argument, 0, 'q'},
  {"verbose", no_argument, 0, 'v'},
  {"help", no_argument, 0, 'h'},
  {"version", no_argument, 0, 'V'},
  {"prob", no_argument, 0, 'p'},
  {"lfold", no_argument, 0, 'l'},
  {"ext", required_argument, 0, 'e'},
  {"compf", required_argument, 0, 0},
  {"pfrali_cw", required_argument, 0, 0},
  {"covar_nc", required_argument, 0, 0},
  {"covar_beta", required_argument, 0, 0},
  {"pmin", required_argument, 0, 0},
  {"ncmax", required_argument, 0, 0},
  {"mountain", no_argument, 0, 'm'},
  {"layers",no_argument, 0, 'L'},
  {NULL, 0, NULL, 0}
};

alidot_options alidot_opt;

extern void PS_dot_plot(char *string, pair_info *pi, pair_info *ppi,
			char *wastlfile);
     
int main(int argc, char *argv[])     
{
  char     *the_names[MAX_NUM_NAMES];   /* sequence names */
  char     *AS[MAX_NUM_NAMES];          /* aligned sequences */
  float    *duck;                       /* entropy of ptable[] */
  /*    char     ali_file[100]; */
  int      n, nn, nl, i, k,cnt, argi, L, maxL;
  pair_info *bpairs, *final_pairs, **BELL;
  int      BPstat[8]={0,0,0,0,0,0,0,0};
  char   **ZEBRA, **NT, *structure;
  int     *NiL, *SiL;
  stack  **ST;
  
  /* defaults */
  alidot_opt.pfrali_cw   = 1.5;
  alidot_opt.covar_nc    = 1;
  alidot_opt.covar_beta  = 1000/(1.98717*(273.15+37)); /* 1/kT in mol/kcal */
  alidot_opt.ncmax       = 2;
  alidot_opt.pmin        = PTHRESH;
  alidot_opt.layers      = 0;  
  argi = decode_switches(argc, argv);
  if (argi==argc-1) {
    alidot_opt.INFILE = fopen(argv[argi], "r");
    if (alidot_opt.INFILE==NULL) nrerror("can't open file");
  } else {
    if (argi<argc) usage(EXIT_FAILURE);
    alidot_opt.INFILE = stdin;
  }
  
  if (strlen(alidot_opt.ext)==0) { 
    if (alidot_opt.p) strcpy(alidot_opt.ext, ".pf");
    else if (alidot_opt.lfold) strcpy(alidot_opt.ext, ".lfold");
    else strcpy(alidot_opt.ext, ".mfe");
  }
  
  nn = read_clustal(alidot_opt.INFILE, AS, the_names);
  
  if (nn==0) {
    fprintf(stderr, "0 aligned sequences\n");
    return(4711);
  }
  n = strlen(AS[0]);
  
  /* now read the structure files ... */
  
  if (alidot_opt.p) bpairs = get_pf_bpairs(the_names, AS);
  else if (alidot_opt.lfold) bpairs = get_lfold_bpairs(the_names, AS);
  else bpairs = get_mfe_bpairs(the_names, AS);
  
  for (i=0; i<nn; i++) free(the_names[i]); /* no longer needed */
  /* we start counting the base pairs from 0 instead of 1 now! */
  
  for (cnt=0; bpairs[cnt].j>0; cnt++);

  /* now fill in the sequence related info and compute the per-position
     ENTropies duck[]     */

  duck =  (float *) space((n+1)*sizeof(float));
  for (i=0; i< cnt; i++) {
    if ( bpairs[i].p>0 ) {
      duck[ bpairs[i].i ] -= bpairs[i].p * log(bpairs[i].p);
      duck[ bpairs[i].j ] -= bpairs[i].p * log(bpairs[i].p);
    }
    /* we're missing the term containing p(unpaired) */
    
    /* test for compensatory mutations */
    
    for (k=0; k<8; k++) bpairs[i].bp[k]=0;
    for (k=0; k<nn; k++)     
      bpairs[i].bp[pair[ENCODE(AS[k][bpairs[i].i])]
		  [ENCODE(AS[k][bpairs[i].j])]]++;  /* da schaugst! */
    
  }
  for (k=0; k<nn; k++) free(AS[k]);  /* no longer needed */
  /* now run through the list again and produce the pseudoentropies */
  
  for (i=0; i< cnt; i++)  
    bpairs[i].ent = duck[bpairs[i].i] + duck[bpairs[i].j] +
      bpairs[i].p * log(bpairs[i].p);  /* S_1 + S_2 - S_12 */
   
  free(duck);
  /* It's also easy to find the most likely pairing partners by 
     running thorugh the index matrix ! - not yet implemented !   */
  
  if (!alidot_opt.want_quiet) fprintf(stderr, "parsed structures\n");
    printf(" %d different pairs.\n", cnt);

  switch (alidot_opt.compf) {
  case 0: /* option not set */
    compare_pair_info = (alidot_opt.p) ? compare_pi_pfrali : compare_pi_alidot;
    if (alidot_opt.lfold) compare_pair_info = compare_pi_cov;
    break;
  case 1:
    compare_pair_info = compare_pi_cov;
    break;
  case 2:
    compare_pair_info = compare_pi_alidot;
    break;
  case 3:
    compare_pair_info = compare_pi_pfrali;
    break;
  }
  /* sort to get most convincing pairs first */
  qsort(bpairs, cnt, sizeof(pair_info), compare_pair_info );
  
  /* for every pair mark whether it conflicts with higher ranked pairs */

  if (alidot_opt.layers) { 
    maxL = pair_layers(bpairs, cnt);
    if (!alidot_opt.want_quiet)
      fprintf(stderr,"Detected basepairs in %d outerplanar layers.\n",maxL);
    if (alidot_opt.want_verbose) dump_pair_file(bpairs,cnt);
  }
  else {
    maxL = 1;
    pair_conficts(bpairs, cnt); 
  }
  
  BELL = believable_pairs(bpairs, n, cnt, maxL);
  NiL  = (int *) space(sizeof(int)*(maxL+1));
  SiL  = (int *) space(sizeof(int)*(maxL+1));
  ST    = (stack **) space(sizeof(stack*)*(maxL+2));
  ZEBRA = (char **) space(sizeof(char*)*(maxL+2));

  for (L=1;L<=maxL;L++) {
    int nb;
    nb=0; while(BELL[L][nb].j!=-1) nb++;
    if (!alidot_opt.want_quiet)
      fprintf(stderr,"L= %3d ... %4d pairs\n",L,nb); 
    NiL[L]=nb;
  }

  for (L=1;L<=maxL;L++) {
    int nst;
    ST[L] = believe2stacks(BELL[L],bpairs,NiL[L],n);
    nst = 0; while(ST[L][nst].best!=-1) nst++;
    SiL[L] = nst;
    ZEBRA[L] = stacks2string(ST[L],n);
  }
  if (!alidot_opt.want_quiet) fprintf(stderr,"Helix tables completed\n");

  /* printout */
  get_BPstat(BELL[1], BPstat, NiL[1]);
  printf("%d believable pairs", NiL[1]);
  for (i=6; i>=1; i--) printf(" %d", BPstat[i]);
  printf("   %d",BPstat[0]);
  if (alidot_opt.layers) printf("   in Layer %d",L);
  printf("\n");
  for (i=0; i< cnt; i++) print_pi(bpairs[i]);
  
  
  for (L=2;L<=maxL;L++) {
    if (alidot_opt.want_verbose) {
      get_BPstat(BELL[L], BPstat, NiL[L]);
      printf("#   %d believable pairs", NiL[L]);
      for (i=6; i>=1; i--) printf(" %d", BPstat[i]);
      printf("   %d",BPstat[0]);
      printf("   in Layer %d",L);
      printf("\n");
      printf("#   %s\n", ZEBRA[L]);
    }
    for (k=1;k<L;k++) { 
      prune_structure(ZEBRA[k],ZEBRA[L]);
    }
    if (alidot_opt.want_verbose) printf("#   %s\n#\n", ZEBRA[L]);
  }

  NT=NULL;
  if (alidot_opt.layers) { 
    if (!alidot_opt.want_quiet) fprintf(stderr,"Raw layers\n");
    if (alidot_opt.want_verbose)
      printf("### Condensed and pruned layer representation\n");
    NT = non_trivial_layers(ZEBRA,maxL);
    i=1; while(NT[i]!=NULL) i++;  nl = i-1;
    if (alidot_opt.want_verbose) for (i=1;i<=nl;i++) printf("%s\n",NT[i]);
    if (!alidot_opt.want_quiet)
      fprintf(stderr,"Got %d condensed layers.\n",nl);
    structure = make_structure(NT);
  }
  else structure = strdup(ZEBRA[1]);

  printf("%s\n",structure); fflush(stdout);
  
  if (alidot_opt.layers)
    final_pairs = make_final_pairs(bpairs,structure);
  else final_pairs = BELL[1];

  i=0;while(final_pairs[i].j>0)i++;
 
  /* Postscript output */ 
  {
    char *fname;
    fname = alidot_opt.p ? "aln_pf_dp.ps" : "aln_dp.ps";
    PS_dot_plot(structure, bpairs, final_pairs, fname);
  }

  /* clean up */
  for (L=0;L<=maxL;L++) {
    free(ZEBRA[L]);
    free(BELL[L]);
    free(ST[L]);
  }
  if (alidot_opt.layers) {
    for (L=0;L<=maxL;L++) {
      if (NT[L]!=NULL) free(NT[L]);
    }
    free(NT);
    free(final_pairs);
  }
  free(ZEBRA); free(BELL); free(NiL); free(ST);
  free(bpairs); free(structure); 
  return 0;
}

/*-------------------------------------------------------------------------*/

PRIVATE pair_info* get_pf_bpairs(char *the_names[], char *AS[]) {
  int i,j, l, k, nn, bp, max_bp, max_ois=0;
  pair_info *bpairs;
  BasePair *ois;

  ois = (BasePair *) space(strlen(AS[0])*2*sizeof(BasePair));
  for (nn=0; the_names[nn]!=NULL; nn++) {
    BasePair *pf_s;
    pf_s = read_bppm(the_names[nn], alidot_opt.ext, AS[nn]);
    if (pf_s==NULL) nrerror("can't read pair prob file");
    /* append to ois */
    ois = (BasePair *) xrealloc(ois, (max_ois+pf_s[0].i+1) * sizeof(BasePair));
    memcpy(&ois[max_ois+1], &pf_s[1], pf_s[0].i*sizeof(BasePair));
    max_ois += pf_s[0].i;
    free(pf_s);
  }

  ois[0].i = max_ois;
  qsort(ois+1, max_ois, sizeof(BasePair), comp_BasePair);

  if (!alidot_opt.want_quiet) fprintf(stderr, "read structures\n");

  l = strlen(AS[0]); max_bp = l*nn;
  bpairs = (pair_info *) space((max_bp+1)*sizeof(pair_info));
  bp = -1;
  for (k=i=j=1; k<=max_ois; k++) {
    if ((ois[k].ai==i)&&(ois[k].aj==j))
      bpairs[bp].p += ois[k].p;
    else {
      bp++;
      if (bp>max_bp) {
	max_bp *= 2;
	bpairs = (pair_info *) xrealloc(bpairs, (max_bp+1)*sizeof(pair_info));
      }
      i = ois[k].ai; j = ois[k].aj;
      bpairs[bp].i = i;
      bpairs[bp].j = j;
      bpairs[bp].p = ois[k].p;
    }
  }
  bp++;
  free(ois);
  for (k=0; k<bp; k++) bpairs[k].p /= nn;
  bpairs = (pair_info *) xrealloc(bpairs,(bp+1)*sizeof(pair_info));
  bpairs[bp].j=0;
  return bpairs;
}

/*-------------------------------------------------------------------------*/


PRIVATE pair_info* get_mfe_bpairs(char *the_names[], char *AS[]) {
  int i, j, l, nn, s, ss, bp, max_pt=32;
  short **ptable;
  pair_info *bpairs;

  l = strlen(AS[0]);

  ptable = (short **) space((max_pt+1)*sizeof(short *));
  for (nn=0; the_names[nn]!=NULL; nn++) {
    char *mfe_s, *Astruct;
    if (nn>max_pt) {
      max_pt *= 2; 
      ptable=(short **) xrealloc(ptable, (max_pt+1)*sizeof(short *));
    }
    mfe_s = read_structure(the_names[nn], alidot_opt.ext);
    if (mfe_s==NULL) nrerror("error reading structures");
    Astruct = expand_struct(AS[nn], mfe_s);
    if (Astruct==NULL) {
      fprintf(stderr, "%s:", the_names[nn]);
      nrerror(" can't read structure");
    } else {
      free(mfe_s);
    }
    ptable[nn] = make_pair_table(Astruct);
    free(Astruct);
  }
  
  if (!alidot_opt.want_quiet) fprintf(stderr, "read structures\n");
  bpairs = (pair_info *) space(l*nn*sizeof(pair_info)/2);
  bp=0;
  for (i=1; i<=l; i++) {
    for (s=0; s<nn; s++) {
      if ((j=ptable[s][i])>i) {
	bpairs[bp].i = i-1;
	bpairs[bp].j = j-1;
	bpairs[bp].p = 1;
	for (ss=s+1; ss<nn; ss++) {
	  if (ptable[ss][i]==j) {
	    bpairs[bp].p++;
	    ptable[ss][i]=0;
	  }
	}
	bpairs[bp++].p /= nn;
      }
    }
  }
  for (s=0; s<nn; s++) free(ptable[s]);
  free(ptable);
  bpairs[bp].j=0;
  bpairs = (pair_info *) xrealloc(bpairs,(bp+1)*sizeof(pair_info));
  return bpairs;
}

PRIVATE pair_info* get_lfold_bpairs(char *the_names[], char *AS[]) {
  int i,j, l, k, nn, bp, max_bp, max_ois=0;
  pair_info *bpairs;
  BasePair *ois;
  
  ois = (BasePair *) space(strlen(AS[0])*2*sizeof(BasePair));
  for (nn=0; the_names[nn]!=NULL; nn++) {
    BasePair *pf_s;
    pf_s = read_lfold(the_names[nn], alidot_opt.ext, AS[nn]);
    if (pf_s==NULL) nrerror("can't read pair lfold file");
    /* append to ois */
    ois = (BasePair *) xrealloc(ois, (max_ois+pf_s[0].i+1) * sizeof(BasePair));
    memcpy(&ois[max_ois+1], &pf_s[1], pf_s[0].i*sizeof(BasePair));
    max_ois += pf_s[0].i;
    free(pf_s);
  }

  ois[0].i = max_ois;
  qsort(ois+1, max_ois, sizeof(BasePair), comp_BasePair);

  if (!alidot_opt.want_quiet) fprintf(stderr, "read structures\n");

  l = strlen(AS[0]); max_bp = l*nn;
  bpairs = (pair_info *) space((max_bp+1)*sizeof(pair_info));
  bp = -1;
  for (k=i=j=1; k<=max_ois; k++) {
    if ((ois[k].ai==i)&&(ois[k].aj==j))
      bpairs[bp].p += ois[k].p;
    else {
      bp++;
      if (bp>max_bp) {
	max_bp *= 2;
	bpairs = (pair_info *) xrealloc(bpairs, (max_bp+1)*sizeof(pair_info));
      }
      i = ois[k].ai; j = ois[k].aj;
      bpairs[bp].i = i;
      bpairs[bp].j = j;
      bpairs[bp].p = ois[k].p;
    }
  }
  bp++;
  free(ois);
  for (k=0; k<bp; k++) bpairs[k].p /= nn;
  bpairs = (pair_info *) xrealloc(bpairs,(bp+1)*sizeof(pair_info));

  return bpairs;
}

/*-------------------------------------------------------------------------*/

PRIVATE int decode_switches (int argc, char **argv) {
  int c;
  int option_index = 0;
  while ((c = getopt_long (argc, argv, 
                           "q"  /* quiet or silent */
                           "v"  /* verbose */
			   "V"  /* version */
                           "h"  /* help */
			   "p"  /* pfrali */
			   "L"  /* layers */
                           "e:", /* extension */
                           long_options, &option_index)) != EOF)
    {
      switch (c)
        {
        case 0:
          if (strcmp(long_options[option_index].name,"compf")==0) 
            if ((sscanf(optarg, "%d", &alidot_opt.compf) == 0)||
		(alidot_opt.compf>3) || (alidot_opt.compf<1))
	      usage(EXIT_FAILURE);

	  if (strcmp(long_options[option_index].name,"pfrali_cw")==0) 
            if ((sscanf(optarg, "%f", &alidot_opt.pfrali_cw) == 0) ||
		(alidot_opt.pfrali_cw>2) || (alidot_opt.pfrali_cw<1))
              usage(EXIT_FAILURE);

	  if (strcmp(long_options[option_index].name,"covar_nc")==0) 
	    if ((sscanf(optarg, "%f", &alidot_opt.covar_nc) == 0) ||
		(alidot_opt.covar_nc<0))
              usage(EXIT_FAILURE);
	  
	  if (strcmp(long_options[option_index].name,"covar_beta")==0) 
            if ((sscanf(optarg, "%f", &alidot_opt.covar_beta) == 0) ||
		(alidot_opt.covar_beta<0))
              usage(EXIT_FAILURE);
	  break;
	case 'q':               /* --quiet */
          alidot_opt.want_quiet = 1;
          break;
        case 'v':               /* --verbose */
          alidot_opt.want_verbose = 1;
          break;
        case 'V':
          printf ("alidot version %s\n%s\n", VERSION, rcsid);
          exit (0);
          
        case 'h':
          usage (0);

	case 'e':
	  if (sscanf(optarg, "%15s", alidot_opt.ext) == 0)
	    usage(EXIT_FAILURE);
	  break;
	case 'p':
	  alidot_opt.p = 1;
	  break;
	case 'l':
	  alidot_opt.lfold = 1;
	  break;
	case 'L':
	  alidot_opt.layers = 1;
	  break;
	default:
	  usage (EXIT_FAILURE);
	}
    }

  return optind;
}
	    

/*-------------------------------------------------------------------------*/
PRIVATE BasePair *read_lfold(char *seqname, char *ext, char *sequ) {
   FILE *lff;
   char fname[100], *line;
   int i, k, n, ii, npair;
   int *transl;
   BasePair * bpl;

   n = strlen(sequ);

   /* first build a translation table. transl[i] = position of 
      i in the aligned sequence(s) */
   
   transl = (int *) space( (n+1)*sizeof(int));
   for (ii=i=0; i<strlen(sequ); i++) {
      if (sequ[i]!=GAPC) {
	 ii++;
	 transl[ii] = i;
      }
   }

   strcpy(fname, seqname);
   strcat(fname, ext);
   lff = fopen(fname, "r");
   if (!lff) {
     fprintf(stderr, "couldn't open file %s\n", fname);
     return NULL;
   }

   npair = 10000;
   bpl = (BasePair *) space ( (npair+1)*sizeof(BasePair) );  

   k=0;
      
   while ((line = get_line(lff))) {
     double E;
     int i, start;
     short *pt;
     char *stru;
     if (line[0] == '>') {
       free(line);
       continue;
     }

     stru = (char *) space(strlen(line)+1);
     if (sscanf(line,"%s (%lf) %d", stru, &E, &start)!=3) {
       break;
     }
     pt = make_pair_table(stru);
     free(stru);
     for (i=1; i<=pt[0]; i++) {
       int ii,j;
       if ((j=pt[i])>i) {
	 k++;
	 ii = transl[i+start-1];
	 j = transl[j+start-1];
	 bpl[k].ai = ii;
	 bpl[k].aj = j;
	 bpl[k].p = 1;
	 if (k==npair) {
	   npair += npair/2 +1;
	   bpl = (BasePair *) xrealloc(bpl, (npair+1)*sizeof(BasePair));
	   if (!bpl) 
	     nrerror("can't allocate bpl");
	 }
       }
     }
     free(pt);
     free(line);
   }
   
   fclose(lff);

   /* remove duplicates */
   {
     int ii;
     qsort(bpl+1, k, sizeof(BasePair), comp_BasePair);
     for (ii=1, i=2; i<=k; i++) {
       if ((bpl[i].ai == bpl[ii].ai) && (bpl[i].aj == bpl[ii].aj)) continue;
       ii++;
       bpl[ii] = bpl[i];
     }
     bpl[0].i = ii;  /* store the number of base pairs in the struct */
   }
   return (bpl);    
}

PRIVATE BasePair *read_bppm(char *seqname, char *ext, char *sequ) {
   FILE *ppf;
   char fname[100], *line;
   int i,j,k,n,ii, npair;
   float p;
   int *transl;
   BasePair * bpl;

   int psfile=0;
   
   n = strlen(sequ);

   /* first build a translation table. transl[i] = position of 
      i in the aligned sequence(s) */
   
   transl = (int *) space( (n+1)*sizeof(int));
   for (ii=i=0; i<strlen(sequ); i++) {
      if (sequ[i]!=GAPC) {
	 ii++;
	 transl[ii] = i;
      }
   }

   strcpy(fname, seqname);
   strcat(fname, ext);
   ppf = fopen(fname, "r");
   if (!ppf) {
      fname[12]='\0';
      strncpy(fname, seqname, 12);   /* RNAfold cuts name after 12 characters */
      strcat(fname, "_dp.ps");
      ppf = fopen(fname, "r");
      if (!ppf) {
	 fprintf(stderr, "couldn't open file %s\n", fname);
	 return NULL;
      }
      else 
	psfile=1;
   }

   npair = 1000;
   bpl = (BasePair *) space ( (npair+1)*sizeof(BasePair) );  

   k=0;
   
   while ((line = get_line(ppf))) {
     if (psfile) {
       int l = strlen(line)-5; 
       /* skip line not of the form:   i j p ubox  */
       if ((line[0]=='%')||(l<5)||(strcmp(line+l," ubox"))) {
	 free(line);
	 continue;
       }
     }
     
     if (sscanf(line,"%d %d %f", &i,&j,&p)!=3) {
       fprintf(stderr,"can't scan line %s\n", line);
       exit(42);
     }
     if (psfile) p=p*p; /* psfile contains sqrt(p) */
     if (p>=alidot_opt.pmin/200.) {
       k++;
       if (i<j) { /* make sure i<j just in case */
	 bpl[k].i = i;
	 bpl[k].j = j;
       } else {
	 bpl[k].i = j;
	 bpl[k].j = i;
       }
       bpl[k].p = p;
       if (k==npair) {
	 npair += npair/2 +1;
	 bpl = (BasePair *) xrealloc(bpl, (npair+1)*sizeof(BasePair));
	 if (!bpl) 
	   nrerror("can't allocate bpl");
       }
     }
     free(line);
   }
   
   fclose(ppf);
   
   bpl[0].i = k;  /* store the number of base pairs in the struct */
   
   for (i=1; i<= k; i++) {
     bpl[i].ai = transl[bpl[i].i];
     bpl[i].aj = transl[bpl[i].j];
   }
   free(transl);
   return (bpl);    
}

/*------------------------------------------------------------------------*/

PRIVATE int comp_BasePair(const void *pi1, const void *pi2) { /* for sort by ai and aj */
  if (((BasePair *) pi1)->ai == ((BasePair *) pi2)->ai)
    return (((BasePair *) pi1)->aj - ((BasePair *) pi2)->aj);
  else
    return (((BasePair *) pi1)->ai - ((BasePair *) pi2)->ai);
}

/*------------------------------------------------------------------------*/
PRIVATE char *read_structure(char *seqname, char *ext) {
   FILE *mfe;
   char fname[100], sname[100], *line, *stru;

   strcpy(fname, seqname);
   strcat(fname, ext);
   mfe = fopen(fname, "r");
   if (!mfe) {
      fprintf(stderr, "couldn't open file %s\n", fname);
      return NULL;
   }

   /* read up to >seq_name line */
   while ((line = get_line(mfe))) {
      if (line[0] == '>') break;
      free(line);
   }
   if (line==NULL) {
      fprintf(stderr, "couldn't find seqence name in %s\n", fname);
      return NULL;
   }
   sscanf(line,">%99s",sname);
   free(line);
   
   /* read line containing structure */
   while ((line = get_line(mfe))) {
      if ((line[0] == '.')||(line[0] == '(')) break;
      free(line);
   }
    if (line==NULL) {
      fprintf(stderr, "couldn't find structure in %s\n", fname);
      return NULL;
   }
   stru = (char *) space(strlen(line)+1);
   sscanf(line, "%s", stru);
   free(line);
   return(stru);
}

PRIVATE char *expand_struct(char *sequ, char *struc) {
   char *estruc;  /* will contain structure with gaps */
   int i, ii;

   estruc = space(strlen(sequ)+1);
   for (i=ii=0; i<strlen(sequ); i++) {
      estruc[i] = (sequ[i]==GAPC) ? GAPC : struc[ii++];
   }
   if (ii != strlen(struc)) {
      fprintf(stderr, "structure and sequence don't match\n");
      return NULL;
   }
   return estruc;
}   

/*------------------------------------------------------------------------*/
PRIVATE void usage(int code) 
{
  fprintf(stderr,
	  "usage: alidot [OPTIONS] [ClustalFile] \n"
	  "OPTIONS\n"
	  "  -e  --ext        extension of structure files\n"
	  "  -p  --prob       analyze pair probabilities (default mfe structures)\n"
	  "  --compf <num>    choose compare function\n"
	  "                   1=covariance, 2=alidot, 3=pfrali\n"
	  "  --pfrali_cw <num> weight of consistent (GC<->GU) mutations,\n"
	  "                   use with --compf=3, 1<=num<=2, default 1.5\n"
	  "  --covar_nc <num> weight of inconsistent sequences in covariance term\n"
	  "                   default 1, use with --compf=1\n"
	  "  --covar_beta <num>  weight of predicted probability in covariance term\n"
	  "                   default 1/kT = 1.62mol/kcal\n"
	  "  --pmin <num>     min probability for a credible pair, default %5.3f\n" 
	  "  --ncmax <num>    max number of inconsistent sequences for a credible pair, default 2\n"
	  "  -L  --layers     compute outerplanar layers\n"
	  "                   this attemps to predict pseudoknots\n"
	  "  -h  --help       print this message\n"
	  "  -q  --quiet      no diagnostics and warnings\n"
	  "  -v  --verbose    print lots of stuff\n"
	  "  -V  --version    print version\n",
	  PTHRESH);
  exit(code);
}

